#!/usr/bin/env python
# coding=utf-8
__version__ = "0.6"
__desc__ = "Convert markdown to html with TOC(table of contents) https://github.com/menduo/mdtree"
from mdtree.main import MdTree, convert_from_file
